unit hiDateConvertor;

interface

uses
  Windows, Kol, Share, Debug;
  
  {.$define HEAP_INT_FIX}

const
  // Sets UnixStartDate to TDateTime of 01/01/1970 in VCL format
  UnixStartDate: TDateTime = 25569;
  
  MAX_DATE = 202751589;
  
  //     Date2StrFmt
  {$ifndef KOL3XX}Date1601 = 584389; {$endif} // 1601-01-01

type
  THIDateConvertor = class(TDebug)
    private
      FResult: TData;
      ST: TSystemTime;
      DateTime: TDateTime;
      FDateTime: string;
      
      procedure HeapIntToXXXX(d: PData; Format: Integer);
      
    public
      _prop_Mode: Byte;
      _prop_Format: string;

      _data_DYear: THI_Event;
      _data_DMonth: THI_Event;
      _data_DDay: THI_Event;
      _data_DHour: THI_Event;
      _data_DMinute: THI_Event;
      _data_DSecond: THI_Event;
      _data_DMilliseconds: THI_Event;

      _data_Data: THI_Event;
      _event_onResult: THI_Event;

      procedure _work_doConvert0  (var _Data: TData; Index: Word); // DateRealToHeapInt
      procedure _work_doConvert1  (var _Data: TData; Index: Word); // HeapIntToDateReal
      procedure _work_doConvert2  (var _Data: TData; Index: Word); // DateVCLHeapInt
      procedure _work_doConvert3  (var _Data: TData; Index: Word); // HeapIntToDateVCL
      procedure _work_doConvert4  (var _Data: TData; Index: Word); // StrFmtToHeapInt
      procedure _work_doConvert5  (var _Data: TData; Index: Word); // HeapIntToStrFmt
      procedure _work_doConvert6  (var _Data: TData; Index: Word); // DateRealToDateVCL
      procedure _work_doConvert7  (var _Data: TData; Index: Word); // DateVCLToDateReal
      procedure _work_doConvert8  (var _Data: TData; Index: Word); // StrFmtToDateReal
      procedure _work_doConvert9  (var _Data: TData; Index: Word); // DateRealToStrFmt
      procedure _work_doConvert10 (var _Data: TData; Index: Word); // StrFmtToDateVCL
      procedure _work_doConvert11 (var _Data: TData; Index: Word); // DateVCLToStrFmt

      procedure _work_doConvert12  (var _Data: TData; Index: Word); // DateUnixToHeapInt
      procedure _work_doConvert13  (var _Data: TData; Index: Word); // HeapIntToDateUnix     
      procedure _work_doConvert14  (var _Data: TData; Index: Word); // DateUnixToDateVCL
      procedure _work_doConvert15  (var _Data: TData; Index: Word); // DateVCLToDateUnix
      procedure _work_doConvert16  (var _Data: TData; Index: Word); // DateUnixToStrFmt               
      procedure _work_doConvert17  (var _Data: TData; Index: Word); // StrFmtToDateUnix
      procedure _work_doConvert18  (var _Data: TData; Index: Word); // DateUnixToDateReal
      procedure _work_doConvert19  (var _Data: TData; Index: Word); // DateRealToDateUnix          

      procedure _var_LeapYear     (var _Data: TData; Index: Word);
      procedure _var_DayOfWeek    (var _Data: TData; Index: Word);
      procedure _var_Year         (var _Data: TData; Index: Word);
      procedure _var_Month        (var _Data: TData; Index: Word);
      procedure _var_Day          (var _Data: TData; Index: Word);
      procedure _var_Hour         (var _Data: TData; Index: Word);
      procedure _var_Minute       (var _Data: TData; Index: Word);
      procedure _var_Second       (var _Data: TData; Index: Word);
      procedure _var_Milliseconds (var _Data: TData; Index: Word);
      procedure _var_Result       (var _Data: TData; Index: Word);
      procedure _var_OffsetUTC     (var _Data: TData; Index: Word);
      procedure _var_OffsetUTCUnix (var _Data: TData; Index: Word);
  end;


var
  OffsetFromUTC: TDateTime; //   (Real/VCL)
  OffsetFromUTCUnix: Integer; // 



implementation





const
  FORMAT_REAL = 0;
  FORMAT_VCL = 1;
  FORMAT_UNIX = 2;
  

procedure THIDateConvertor.HeapIntToXXXX(d: PData; Format: Integer);
begin
  FillChar(ST, SizeOf(ST), 0);

  ST.wYear :=         ReadInteger(d^, _data_DYear, 0);
  ST.wMonth :=        ReadInteger(d^, _data_DMonth, 0);
  ST.wDay :=          ReadInteger(d^, _data_DDay, 0);
  ST.wHour :=         ReadInteger(d^, _data_DHour, 0);
  ST.wMinute :=       ReadInteger(d^, _data_DMinute, 0);
  ST.wSecond :=       ReadInteger(d^, _data_DSecond, 0);
  ST.wMilliseconds := ReadInteger(d^, _data_DMilliseconds, 0);

  if ST.wYear = 0 then
  begin
    case Format of
      FORMAT_VCL:
        ST.wYear := 1899;
      FORMAT_UNIX:
        ST.wYear := 1970;
      else
        ST.wYear := 1; // FORMAT_REAL
    end;
  end;

  if ST.wMonth = 0 then
  begin
    if Format = FORMAT_VCL then
      ST.wMonth := 12
    else
      ST.wMonth := 1;
  end;

  if ST.wDay = 0 then
  begin
    if Format = FORMAT_VCL then
      ST.wDay := 30
    else
      ST.wDay := 1;
  end;

  SystemTime2DateTime(ST, DateTime);

  //   SystemTime2DateTime
  {$ifdef HEAP_INT_FIX} if Format = FORMAT_REAL then DateTime := DateTime - 1; {$endif}
end;

procedure THIDateConvertor._work_doConvert0; // DateRealToHeapInt
begin
  DateTime := ReadReal(_Data,_data_Data,0);
  if DateTime > MAX_DATE then exit;
  FillChar(ST, SizeOf(ST), 0);
  DateTime2SystemTime(DateTime {$ifdef HEAP_INT_FIX} + 1 {$endif}, ST);
  dtNull(FResult);
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert1; // HeapIntToDateReal
begin
  HeapIntToXXXX(@_Data, FORMAT_REAL);
  dtReal(FResult, DateTime);
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert2; // DateVCLHeapInt
begin
  DateTime := ReadReal(_Data, _data_Data, 0) + VCLDate0;
  if DateTime > MAX_DATE then exit;
  FillChar(ST, SizeOf(ST), 0);
  DateTime2SystemTime(DateTime, ST);
  dtNull(FResult);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert3; // HeapIntToDateVCL
begin
  HeapIntToXXXX(@_Data, FORMAT_VCL);
  dtReal(FResult, DateTime - VCLDate0);
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert4; // StrFmtToHeapInt
begin
  FDateTime := ReadString(_Data, _data_Data, '');
  DateTime := Str2DateTimeFmt(_prop_Format, FDateTime);
  if DateTime > MAX_DATE then exit;
  FillChar(ST, SizeOf(ST), 0);
  DateTime2SystemTime(DateTime, ST);
  dtNull(FResult);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);  
end;

procedure THIDateConvertor._work_doConvert5; // HeapIntToStrFmt
begin
  HeapIntToXXXX(@_Data, FORMAT_REAL);
  dtString(FResult, Time2StrFmt(Date2StrFmt(_prop_Format, DateTime), DateTime));
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert6; // DateRealToDateVCL
begin
  dtReal(FResult, ReadReal(_Data, _data_Data, 0) - VCLDate0);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);   
end;

procedure THIDateConvertor._work_doConvert7; // DateVCLToDateReal
begin
  dtReal(FResult, ReadReal(_Data,_data_Data,0) + VCLDate0);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);  
end;

procedure THIDateConvertor._work_doConvert8; // StrFmtToDateReal
begin
  FDateTime := ReadString(_Data, _data_Data, '');
  DateTime := Str2DateTimeFmt(_prop_Format, FDateTime);
  if DateTime > MAX_DATE then exit;
  dtReal(FResult, DateTime);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);   
end;

procedure THIDateConvertor._work_doConvert9; // DateRealToStrFmt
begin
  DateTime := ReadReal(_Data, _data_Data, 0);
  if DateTime > MAX_DATE then exit;

  if DateTime < Date1601 then DateTime := Frac(DateTime) + Date1601;

  dtString(FResult, Date2StrFmt(Time2StrFmt(_prop_Format, DateTime), DateTime));
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert10; // StrFmtToDateVCL
begin
  FDateTime := ReadString(_Data, _data_Data, '');
  DateTime := Str2DateTimeFmt(_prop_Format, FDateTime);

  //    DateReal  = 0, ,   
  //    DateVCL   = 0   30.12.1899.
  if Trunc(DateTime) <> 0 then DateTime := DateTime - VCLDate0;

  if DateTime > MAX_DATE then exit;

  dtReal(FResult, DateTime);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);   
end;

procedure THIDateConvertor._work_doConvert11; // DateVCLToStrFmt
begin
  DateTime := ReadReal(_Data, _data_Data, 0) + VCLDate0;
  if DateTime > MAX_DATE then exit;

  if DateTime < Date1601 then DateTime := Frac(DateTime) + Date1601;

  dtString(FResult, Time2StrFmt(Date2StrFmt(_prop_Format, DateTime), DateTime));
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert12; // DateUnixToHeapInt
begin
  DateTime := (Trunc(ReadReal(_Data, _data_Data, 0)) / SecsPerDay) + UnixStartDate  + VCLDate0;
  if DateTime > MAX_DATE then exit;
  FillChar(ST, SizeOf(ST), 0);
  DateTime2SystemTime(DateTime, ST);
  dtNull(FResult);
  _hi_CreateEvent(_Data, @_event_onResult, FResult); 
end;

procedure THIDateConvertor._work_doConvert13; // HeapIntToDateUnix     
begin
  HeapIntToXXXX(@_Data, FORMAT_UNIX);
  dtReal(FResult, Round((DateTime - UnixStartDate - VCLDate0) * SecsPerDay));
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert14; // DateUnixToDateVCL
begin
  dtReal(FResult, (Trunc(ReadReal(_Data, _data_Data, 0)) / SecsPerDay) + UnixStartDate);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert15; // DateVCLToDateUnix
begin
  dtReal(FResult, Round((ReadReal(_Data, _data_Data, 0) - UnixStartDate) * SecsPerDay));
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert16; // DateUnixToStrFmt               
begin
  DateTime := ((Trunc(ReadReal(_Data, _data_Data, 0)) / SecsPerDay) + UnixStartDate  + VCLDate0);
  if DateTime > MAX_DATE then exit;

  if DateTime < Date1601 then DateTime := Frac(DateTime) + Date1601;

  dtString(FResult, Time2StrFmt(Date2StrFmt(_prop_Format, DateTime), DateTime));
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert17; // StrFmtToDateUnix
begin
  FDateTime := ReadString(_Data, _data_Data, '');
  DateTime := Str2DateTimeFmt(_prop_Format, FDateTime);
  if DateTime > MAX_DATE then exit;

  //    DateReal  = 0, ,   
  //    DateUnix   = 0   01.01.1970.
  if Trunc(DateTime) <> 0 then DateTime := DateTime - UnixStartDate - VCLDate0;
  DateTime := Round(DateTime * SecsPerDay);

  //  32- .  ,     dtReal
  //if DateTime < UnixMinDate then DateTime := UnixMinDate;
  //dtInteger(FResult, Round(DateTime));

  dtReal(FResult, DateTime);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert18; // DateUnixToDateReal
begin
  dtReal(FResult, (Trunc(ReadReal(_Data, _data_Data, 0)) / SecsPerDay) + UnixStartDate + VCLDate0);
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._work_doConvert19; // DateRealToDateUnix  
begin
  dtReal(FResult, Round((ReadReal(_Data, _data_Data, 0) - UnixStartDate - VCLDate0) * SecsPerDay));
  _hi_CreateEvent(_Data, @_event_onResult, FResult);
end;

procedure THIDateConvertor._var_Result;
begin
  _Data := FResult; 
end;

procedure THIDateConvertor._var_LeapYear;
begin
  dtInteger(_Data, Ord(IsLeapYear(ST.wYear)));
end;

procedure THIDateConvertor._var_DayOfWeek;    begin dtInteger(_Data, DayOfWeek(DateTime)); end;
procedure THIDateConvertor._var_Year;         begin dtInteger(_Data, ST.wYear); end;
procedure THIDateConvertor._var_Month;        begin dtInteger(_Data, ST.wMonth); end;
procedure THIDateConvertor._var_Day;          begin dtInteger(_Data, ST.wDay); end;
procedure THIDateConvertor._var_Hour;         begin dtInteger(_Data, ST.wHour); end;
procedure THIDateConvertor._var_Minute;       begin dtInteger(_Data, ST.wMinute); end;
procedure THIDateConvertor._var_Second;       begin dtInteger(_Data, ST.wSecond); end;
procedure THIDateConvertor._var_Milliseconds; begin dtInteger(_Data, ST.wMilliseconds); end;

procedure THIDateConvertor._var_OffsetUTC;     begin dtReal(_Data, OffsetFromUTC); end;
procedure THIDateConvertor._var_OffsetUTCUnix; begin dtInteger(_Data, OffsetFromUTCUnix); end;


//     UTC (GMT)  
// (     )

function GetOffsetFromUTC: Integer;
var
  iBias: Integer;
  tmez: TTimeZoneInformation;
begin
  iBias := 0;

  case GetTimeZoneInformation(tmez) of
    TIME_ZONE_ID_UNKNOWN  :
       iBias := tmez.Bias;
    TIME_ZONE_ID_DAYLIGHT :
    begin
      iBias := tmez.Bias;
      if tmez.DaylightDate.wMonth <> 0 then
      begin
        iBias := iBias + tmez.DaylightBias;
      end;
    end;
    TIME_ZONE_ID_STANDARD :
    begin
      iBias := tmez.Bias;
      if tmez.StandardDate.wMonth <> 0 then
      begin
        iBias := iBias + tmez.StandardBias;
      end;
    end
  end;
    
  {The GetTimeZone function returns values oriented towards converting
   a GMT time into a local time.  We wish to do the opposite by returning
   the difference between the local time and GMT.  So I just make a positive
   value negative and leave a negative value as positive}

  Result := -iBias;
end;

initialization
  OffsetFromUTCUnix := GetOffsetFromUTC * 60;
  OffsetFromUTC := OffsetFromUTCUnix / SecsPerDay;

end.